/*
   CD Database Application

   Beginning Linux Programming

   Version: Terminals

   Copyright (c) 1996,2007 Wrox Press

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Fee Software Foundation; either version 2 of the License, or (at
   your option) any later version.

   This program is distributed in the hopes that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc. 675 Mass Ave, Cambridge, MA 02139, USA.

 */

/*
 Tuto verzi aplikace s databz sbrky CD jsme napsali s vyuitm informac, probranch
 v pedchzejcch kapitolch. Jej funkce jsme odvodili z pvodn shellov aplikace
 (skriptu) z kapitoly 2. Ped implementac v C jsme jej nvrh nijak neupravovali, a
 proto je v n stle "ctit" spousta shellovch obrat. Poznamenejme, e slabinou
 tto implementace je tak nkolik pomrn vznamnch omezen, kter vyeme v dalch verzch aplikace.
*/

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <curses.h>


#define MAX_STRING   80    /* Nejdel ppustn odpov */
#define MAX_ENTRY  1024    /* Nejdel ppustn poloka v databzi */

#define MESSAGE_LINE  6    /* Na tento dek se vypisuj rzn zprvy */
#define ERROR_LINE   22    /* Na tento dek se vypisuj chyby */
#define Q_LINE       20    /* Na tento dek se vypisuj otzky */
#define PROMPT_LINE  18    /* Na tento dek se vypisuj vzvy */

/* 
Dle budeme v aplikaci potebovat jist globln promnn. V promnn current_cd bude
 uloen vdy titul aktulnho CD, s nm momentln pracujeme. Po inicializaci je 
jeho prvn znak nulov, co znamen, e nen vybrno dn CD. Piazen \0 nen 
pln nezbytn nutn, ale explicitn inicializace je v zsad rozumn. Promnn 
current_cat pak zaznamenv katalogov slo aktulnho CD. */

static char current_cd[MAX_STRING] = "\0";
static char current_cat[MAX_STRING];

// Nyn deklarujeme nzvy soubor; pro jednoduchost budou v tto verzi vechny nzvy soubor pevn, a to i nzev doasnho souboru.

const char *title_file = "title.cdb";
const char *tracks_file = "tracks.cdb";
const char *temp_file = "cdb.tmp";


/* 	Pot nsleduj prototypy funkc */
void clear_all_screen(void);
void get_return(void);
int get_confirm(void);
int getchoice(char *greet, char *choices[]);
void draw_menu(char *options[], int highlight, int start_row, int start_col);
void insert_title(char *cdtitle);
void get_string(char *string);
void add_record(void);
void count_cds(void);
void find_cd(void);
void list_tracks(void);
void remove_tracks(void);
void remove_cd(void);
void update_cd(void);


/* Ne se podvme na implementaci tchto funkc, musme jet definovat jist struktury 
pro nabdky (pole text voleb). Pi vybrn urit volby se vrt vdy prvn znak textu 
nabdky; z volby "a - pidat nov CD" se napklad vrt psmeno "a". Druh, rozen 
nabdka se zobrazuje v ppad, e je vybrno urit CD:
*/

char *main_menu[] =
{
    "a - pidat nov CD",
    "f - hledat CD",
    "c - poet CD a stop v katalogu",
    "q - konec",
    0,
};

char *extended_menu[] =
{
    "a - pidat nov CD",
    "f - hledat CD",
    "c - poet CD a stop v katalogu",
    "l - vypsat stopy na aktulnm CD",
    "r - smazat aktuln CD",
    "u - aktualizovat informace o stopch",
    "q - konec",
    0,
};


int main()
{
    int choice;
    initscr();
    do {
        choice = getchoice("Volby:",
                 current_cd[0] ? extended_menu : main_menu);
        switch (choice) {
        case 'q':
            break;
        case 'a':
            add_record();
            break;
        case 'c':
            count_cds();
            break;
        case 'f':
            find_cd();
            break;
        case 'l':
            list_tracks();
            break;
        case 'r':
            remove_cd();
            break;
        case 'u':
            update_cd();
            break;
        }
    } while (choice != 'q');
    endwin();
    exit(EXIT_SUCCESS);
}

/* Nejdleitj funkc tto sti je getchoice, kterou pmo vol funkce main. V parametrech 
pebr pozdrav greet a pole voleb choices s ukazatelem na hlavn nebo rozenou nabdku 
(podle toho, jestli je v programu vybrno njak CD). Pedvn pole main_menu nebo 
extended_menu jsme vidli ped chvl ve funkci main:
*/

int getchoice(char *greet, char *choices[])
{
    static int selected_row = 0;
    int max_row = 0;
    int start_screenrow = MESSAGE_LINE, start_screencol = 10;
    char **option;
    int selected;
    int key = 0;

    option = choices;
    while (*option) {
        max_row++;
        option++;
    }

    /* ochrana proti zkrcen nabdky v ppad smazn CD */

    if (selected_row >= max_row)
        selected_row = 0;
    clear_all_screen();
    mvprintw(start_screenrow - 2, start_screencol, greet);
    keypad(stdscr, TRUE);
    cbreak();
    noecho();
    key = 0;

    while (key != 'q' && key != KEY_ENTER && key != '\n') {
        if (key == KEY_UP) {
            if (selected_row == 0)
                selected_row = max_row - 1;
            else
                selected_row--;
        }
        if (key == KEY_DOWN) {
        if (selected_row == (max_row - 1))
            selected_row = 0;
        else
            selected_row++;
        }
        selected = *choices[selected_row];
        draw_menu(choices, selected_row, start_screenrow,
                  start_screencol);
        key = getch();
    }
    keypad(stdscr, FALSE);
    nocbreak();
    echo();

        if (key == 'q')
    selected = 'q';

    return (selected);
}

// Dle si vimneme dvou loklnch funkc, kter vyvolv funkce getchoice: 
// jsou to clear_all_screen a draw_menu. Nejprve se seznmme s funkc draw_menu:
void draw_menu(char *options[], int current_highlight,
               int start_row, int start_col)
{
    int current_row = 0;
    char **option_ptr;
    char *txt_ptr;

    option_ptr = options;
    while (*option_ptr) {
        if (current_row == current_highlight) attron(A_STANDOUT);
        txt_ptr = options[current_row];
        txt_ptr++;
        mvprintw(start_row + current_row, start_col, "%s", txt_ptr);
        if (current_row == current_highlight) attroff(A_STANDOUT);
        current_row++;
        option_ptr++;
    }
    mvprintw(start_row + current_row + 3, start_col,
            "Posute zvraznn vbr a stisknte Enter ");
    refresh();
}

// A posledn funkc tto sti je clear_all_screen, kter - jak jist uhodnete - 
// vymae obrazovku a pepe titul okna. Pokud je vybrno njak CD, vype i tuto informaci:
void clear_all_screen()
{
    clear();
    mvprintw(2, 20, "%s", "Aplikace s databz CD");
    if (current_cd[0]) {
        mvprintw(ERROR_LINE, 0, "Aktuln CD: %s: %s\n",
                 current_cat, current_cd);
    }
    refresh();
}

// get_return, kter vype text vzvy a pete nvrat vozku, piem jin znaky ignoruje:
void get_return()
{
    int ch;

    mvprintw(23, 0, "%s", " Stisknte Enter ");
    refresh();
    while ((ch = getchar()) != '\n' && ch != EOF);
}

// Dal funkce, get_confirm, vype uivateli vzvu a nate od nj potvrzen. Pesnji nate 
// od uivatele vstupn etzec a prvn znak porovn s psmenem A nebo a; je-li prvn znak jin, potvrzen se povauje za nespn:
int get_confirm()
{
    int confirmed = 0;
    char first_char;

    mvprintw(Q_LINE, 5, "Jste si jist? ");
    clrtoeol();
    refresh();

    cbreak();
    first_char = getch();
    if (first_char == 'Y' || first_char == 'y' || first_char == 'A' || first_char == 'a') {
    confirmed = 1;
    }
    nocbreak();

    if (!confirmed) {
        mvprintw(Q_LINE, 1, "Zrueno");
        clrtoeol();
        refresh();
        sleep(1);
    }
    return confirmed;
}

// funkce insert_title, jejm kolem je pidn titulu do databze - to znamen, e zape etzec titulu na konec souboru titul:
void insert_title(char *cdtitle)
{
    FILE *fp = fopen(title_file, "a");
    if (!fp) {
        mvprintw(ERROR_LINE, 0, "Nemohu otevt databzi titul CD");
    } else {
        fprintf(fp, "%s\n", cdtitle);
        fclose(fp);
    }
}

// Funkce get_string vype vzvu a pete etzec z aktuln pozice na obrazovce. 
// Zrove odstran ppadn koncov znak novho dku:
void get_string(char *string)
{
    int len;

    wgetnstr(stdscr, string, MAX_STRING);
    len = strlen(string);
    if (len > 0 && string[len - 1] == '\n')
        string[len - 1] = '\0';
}

// Takto pidme do databze zznam novho CD:
void add_record()
{
    char catalog_number[MAX_STRING];
    char cd_title[MAX_STRING];
    char cd_type[MAX_STRING];
    char cd_artist[MAX_STRING];
    char cd_entry[MAX_STRING];

    int screenrow = MESSAGE_LINE;
    int screencol = 10;

    clear_all_screen();
    mvprintw(screenrow, screencol, "Zadejte informace o novm CD");
    screenrow += 2;

    mvprintw(screenrow, screencol, "Katalogov slo: ");
    get_string(catalog_number);
    screenrow++;

    mvprintw(screenrow, screencol, "Titul CD: ");
    get_string(cd_title);
    screenrow++;

    mvprintw(screenrow, screencol, "Typ CD: ");
    get_string(cd_type);
    screenrow++;

    mvprintw(screenrow, screencol, "Umlec: ");
    get_string(cd_artist);
    screenrow++;

    mvprintw(PROMPT_LINE-2, 5, "Pidvm tuto novou poloku:");
    sprintf(cd_entry, "%s,%s,%s,%s",
            catalog_number, cd_title, cd_type, cd_artist);
    mvprintw(PROMPT_LINE, 5, "%s", cd_entry);
    refresh();

    move(PROMPT_LINE, 0);
    if (get_confirm()) {
        insert_title(cd_entry);
        strcpy(current_cd, cd_title);
        strcpy(current_cat, catalog_number);
    }
}

// Pro kadho sbratele je nejdleitj vdt, kolik pslunch "cennost" ve sbrce m. 
// Tohoto kolu se obdivuhodn zhost nsledujc funkce, kter projde celou databzi a spot tituly i stopy:
void count_cds()
{
    FILE *titles_fp, *tracks_fp;
    char entry[MAX_ENTRY];
    int titles = 0;
    int tracks = 0;

    titles_fp = fopen(title_file, "r");
    if (titles_fp) {
        while (fgets(entry, MAX_ENTRY, titles_fp))
            titles++;
        fclose(titles_fp);
    }

    tracks_fp = fopen(tracks_file, "r");
    if (tracks_fp) {
        while (fgets(entry, MAX_ENTRY, tracks_fp))
            tracks++;
        fclose(tracks_fp);
    }
    mvprintw(ERROR_LINE, 0,
        "Databze obsahuje %d titul a na nich celkem %d stop.",
        titles, tracks);
    get_return();
}

// Pedstavte si, e jste ztratili booklet ze svho nejoblbenjho CD - ale nepropadejte panice! 
// Protoe jste jist vechny informace peliv podili do databze, dky funkci find_cd nyn 
// snadno najdete potebn seznam stop. Tato funkce si vyd podetzec, kter bude v databzi 
// hledat, a nakonec nalezen titul zape do globln promnn current_cd:
void find_cd()
{
    char match[MAX_STRING], entry[MAX_ENTRY];
    FILE *titles_fp;
    int count = 0;
    char *found, *title, *catalog;

    mvprintw(Q_LINE, 0, "Zadejte etzec, kter se bude vyhledvat mezi tituly CD: ");
    get_string(match);

    titles_fp = fopen(title_file, "r");
    if (titles_fp) {
        while (fgets(entry, MAX_ENTRY, titles_fp)) {

            /* Pesko katalogov slo */
            catalog = entry;
            if (found = strstr(catalog, ",")) {
                *found = '\0';
                title = found + 1;

                /* Odstran nejbli rku v poloce a tm 
                   ponech jen samotn titul */
                if (found = strstr(title, ",")) {
                    *found = '\0';

                    /* Nyn zkontroluje, jestli byl nalezen shodn podetzec */
                    if (found = strstr(title, match)) {
                        count++;
                        strcpy(current_cd, title);
                        strcpy(current_cat, catalog);
                    }
                }
            }
        }
        fclose(titles_fp);
    }
    if (count != 1) {
        if (count == 0) {
            mvprintw(ERROR_LINE, 0, "Je mi lto, pslun CD nebylo nalezeno. ");
        }
        if (count > 1) {
            mvprintw(ERROR_LINE, 0,
                    "Je mi lto, zadn nen jednoznan: nalezeno %d CD. ", count);
        }
        current_cd[0] = '\0';
        get_return();
    }
}
// etzec catalog ukazuje sice na vt pole ne current_cat a teoreticky by tak mohl zpsobit 
// pepsn pamti, tomu ale zabrnme pomoc kontroly ve voln fgets.


// remove_tracks, kter odstrauje stopy z aktulnho CD. Tuto funkci vyuvaj funkce update_cd a remove_cd:
void remove_tracks()
{
    FILE *tracks_fp, *temp_fp;
    char entry[MAX_ENTRY];
    int cat_length;

    if (current_cd[0] == '\0')
        return;

    cat_length = strlen(current_cat);

    tracks_fp = fopen(tracks_file, "r");
    if (tracks_fp == (FILE *)NULL) return;
    temp_fp = fopen(temp_file, "w");

    while (fgets(entry, MAX_ENTRY, tracks_fp)) {
        /* Porovnat katalogov slo a poloku koprovat jen pi neshod */
        if (strncmp(current_cat, entry, cat_length) != 0)
            fputs(entry, temp_fp);
    }
    fclose(tracks_fp);
    fclose(temp_fp);

    /* Odstranit pvodn soubor stop a pejmenovat doasn soubor */
    unlink(tracks_file);
    rename(temp_file, tracks_file);
}

// funkce volan z funkce main m nzev remove_cd:
void remove_cd()
{
    FILE *titles_fp, *temp_fp;
    char entry[MAX_ENTRY];
    int cat_length;

    if (current_cd[0] == '\0')
        return;

    clear_all_screen();
    mvprintw(PROMPT_LINE, 0, "Budu odstraovat CD %s: %s. ",
             current_cat, current_cd);

    if (!get_confirm())
        return;

    cat_length = strlen(current_cat);

    /* Zkoprovat soubor titul do doasnho souboru a toto CD ignorovat */
    titles_fp = fopen(title_file, "r");
    temp_fp = fopen(temp_file, "w");

    while (fgets(entry, MAX_ENTRY, titles_fp)) {
        /* Porovnat katalogov slo a poloku koprovat jen pi neshod */
        if (strncmp(current_cat, entry, cat_length) != 0)
            fputs(entry, temp_fp);
    }
    fclose(titles_fp);
    fclose(temp_fp);

    /* Odstranit pvodn soubor titul a pejmenovat doasn soubor */
    unlink(title_file);
    rename(temp_file, title_file);

    /* Nyn tot pro soubor stop */
    remove_tracks();

    /* Aktuln vybrno nen dn CD */
    current_cd[0] = '\0';
}

#define BOXED_LINES 11
#define BOXED_ROWS 60
#define BOX_LINE_POS 8
#define BOX_ROW_POS 2

// Dle budeme potebovat vypsat na obrazovku seznam stop vybranho CD. Zde meme vyut konstanty 
// #define, kter jsme pro podokna nadefinovali u funkce update_cd:
void list_tracks()
{
    FILE *tracks_fp;
    char entry[MAX_ENTRY];
    int cat_length;
    int lines_op = 0;
    WINDOW *track_pad_ptr;
    int tracks = 0;
    int key;
    int first_line = 0;

    if (current_cd[0] == '\0') {
        mvprintw(ERROR_LINE, 0, "Nejprve muste vybrat CD. ");
        get_return();
        return;
    }
    clear_all_screen();
    cat_length = strlen(current_cat);

    /* Nejprve zjistme poet stop aktulnho CD */
    tracks_fp = fopen(tracks_file, "r");
    if (!tracks_fp)
        return;
    while (fgets(entry, MAX_ENTRY, tracks_fp)) {
        if (strncmp(current_cat, entry, cat_length) == 0)
            tracks++;
    }
    fclose(tracks_fp);

    /* Vytvome novou plochu ; i kdy je na CD teba jen 
       jedna stopa, mus bt plocha  dostaten velk, aby 
       nsledujc voln prefresh() bylo vdy platn. */
    track_pad_ptr = newpad(tracks + 1 + BOXED_LINES, BOXED_ROWS + 1);
    if (!track_pad_ptr)
        return;

    tracks_fp = fopen(tracks_file, "r");
    if (!tracks_fp)
        return;

    mvprintw(4, 0, "Vpis stop na CD\n");

    /* zpis informac o stopch do plochy  */
    while (fgets(entry, MAX_ENTRY, tracks_fp)) {

        /* Porovnat katalogov slo a vypsat zbytek poloky */
        if (strncmp(current_cat, entry, cat_length) == 0) {
            mvwprintw(track_pad_ptr, lines_op++, 0, "%s",
            entry + cat_length + 1);
        }
    }
    fclose(tracks_fp);

    if (lines_op > BOXED_LINES) {
        mvprintw(MESSAGE_LINE, 0,
                "Vpis mete posouvat ipkami, Enter nebo q pro ukonen");
    } else {
        mvprintw(MESSAGE_LINE, 0, "Enter nebo q pro ukonen");
    }

    wrefresh(stdscr);
    keypad(stdscr, TRUE);
    cbreak();
    noecho();
    key = 0;

    while (key != 'q' && key != KEY_ENTER && key != '\n') {
        if (key == KEY_UP) {
            if (first_line > 0)
                first_line--;
        }
        if (key == KEY_DOWN) {
            if (first_line + BOXED_LINES + 1 < tracks)
                first_line++;
        }

        /* nyn na obrazovce vykreslme pslunou st plochy  */
        prefresh(track_pad_ptr, first_line, 0,
                BOX_LINE_POS, BOX_ROW_POS,
                BOX_LINE_POS + BOXED_LINES, BOX_ROW_POS + BOXED_ROWS);
        key = getch();
    }

    delwin(track_pad_ptr);
    keypad(stdscr, FALSE);
    nocbreak();
    echo();
}

/* Pejdme k funkcm, pomoc nich main provd dal manipulaci se soubory. Nejprve se podvme
 na funkci update_cd, kter pracuje s posuvnm podoknem s rmekem a potebuje ke sv innosti
 jist konstanty - ty nadefinujeme globln, protoe je pozdji vyuijeme jet ve funkci list_tracks.
*/

// Funkce update_cd slou k optovnmu zadvn stop aktulnho CD; po odstrann 
// pvodnho zznamu stop vyzve uivatele k zadn novch informac:
void update_cd()
{
    FILE *tracks_fp;
    char track_name[MAX_STRING];
    int len;
    int track = 1;
    int screen_line = 1;
    WINDOW *box_window_ptr;
    WINDOW *sub_window_ptr;

    clear_all_screen();
    mvprintw(PROMPT_LINE, 0, "Nov zadn stop pro CD. ");
    if (!get_confirm())
    return;
    move(PROMPT_LINE, 0);
    clrtoeol();

    remove_tracks();

    mvprintw(MESSAGE_LINE, 0, "Zadn przdnho dku znamen konec");

    tracks_fp = fopen(tracks_file, "a");
/* V tomto mst vpis na okamik perume a v malm intermezzu upozornme na zpsob zadvn
 informac do posuvnho okna s rmekem. Podstatnm "trikem" je zde vytvoen dvou podoken - 
 prvn ormujeme a tsn pod nm vytvome druh, posuvn podokno.
*/
    box_window_ptr = subwin(stdscr, BOXED_LINES + 2, BOXED_ROWS + 2,
                            BOX_LINE_POS - 1, BOX_ROW_POS - 1);
    if (!box_window_ptr)
        return;

    box(box_window_ptr, ACS_VLINE, ACS_HLINE);

    sub_window_ptr = subwin(stdscr, BOXED_LINES, BOXED_ROWS,
                            BOX_LINE_POS, BOX_ROW_POS);
    if (!sub_window_ptr)
        return;

    scrollok(sub_window_ptr, TRUE);
    werase(sub_window_ptr);
    touchwin(stdscr);

    do {
        mvwprintw(sub_window_ptr, screen_line++, BOX_ROW_POS + 2,
                  "Stopa %d: ", track);
        clrtoeol();
        refresh();
        wgetnstr(sub_window_ptr, track_name, MAX_STRING);
        len = strlen(track_name);
        if (len > 0 && track_name[len - 1] == '\n')
            track_name[len - 1] = '\0';
        if (*track_name)
            fprintf(tracks_fp, "%s,%d,%s\n", current_cat, track, track_name);
        track++;
        if (screen_line > BOXED_LINES - 1) {
            /* nyn zahjme posouvn */
            scroll(sub_window_ptr);
            screen_line--;
        }
    } while (*track_name);
    delwin(sub_window_ptr);
    fclose(tracks_fp);
}

