#define _XOPEN_SOURCE

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <string.h>

#include <ndbm.h>
/* V nkterch distribucch musme ve uvedenou deklaraci zmnit na gdbm-ndbm.h */

#include "cd_data.h"

#define CDC_FILE_BASE "cdc_data"
#define CDT_FILE_BASE "cdt_data"
#define CDC_FILE_DIR "cdc_data.dir"
#define CDC_FILE_PAG "cdc_data.pag"
#define CDT_FILE_DIR "cdt_data.dir"
#define CDT_FILE_PAG "cdt_data.pag"

//	V tchto dvou globlnch promnnch budeme sledovat aktuln databzi:
static DBM *cdc_dbm_ptr = NULL;
static DBM *cdt_dbm_ptr = NULL;

/* 
	Pi vchozm tvaru voln oteve funkce database_initialize existujc databzi, 
	ale pokud do n pedme nenulov (tedy "pravdiv") parametr new_database, 
	vydme si od n vytvoen nov przdn databze a tm i odstrann ppadn 
	stvajc databze. Probhne-li inicializace databze spn, inicializujeme 
	tak dvojici databzovch ukazatel, kter indikuj, e je databze sprvn otevena:
*/

int database_initialize(const int new_database)
{
    int open_mode = O_CREAT | O_RDWR;

    /* Pokud je otevena njak stvajc databze, uzaveme ji */
    if (cdc_dbm_ptr) dbm_close(cdc_dbm_ptr);
    if (cdt_dbm_ptr) dbm_close(cdt_dbm_ptr);

    if (new_database) {
        /* Odstrann starch soubor */
        (void) unlink(CDC_FILE_PAG);
        (void) unlink(CDC_FILE_DIR);
        (void) unlink(CDT_FILE_PAG);
        (void) unlink(CDT_FILE_DIR);
    }

    /* Otevt nov soubory, podle poteby je i vytvoit */
    cdc_dbm_ptr = dbm_open(CDC_FILE_BASE, open_mode, 0644);
    cdt_dbm_ptr = dbm_open(CDT_FILE_BASE, open_mode, 0644);

    if (!cdc_dbm_ptr || !cdt_dbm_ptr) {
        fprintf(stderr, "Nemohu vytvoit databzi\n");
        cdc_dbm_ptr = cdt_dbm_ptr = NULL;
        return (0);
    }
    return (1);
}

/* 
	Funkce database_close jednodue uzave databzi (pokud byla otevena) a do obou 
	databzovch ukazatel zape hodnotu NULL, kter znamen, e momentln nen otevena dn databze:
*/

void database_close(void)
{
    if (cdc_dbm_ptr) dbm_close(cdc_dbm_ptr);
    if (cdt_dbm_ptr) dbm_close(cdt_dbm_ptr);

    cdc_dbm_ptr = cdt_dbm_ptr = NULL;
}

/*
	Nsleduje funkce, kter podle pedanho ukazatele na etzec s katalogovm textem 
	nate jednu poloku katalogu. Nen-li poloka nalezena, obsahuje navrcen 
	struktura przdn pole katalogu:
*/
cdc_entry get_cdc_entry(const char *cd_catalog_ptr)
{
    cdc_entry entry_to_return;
    char entry_to_find[CAT_CAT_LEN + 1];
    datum local_data_datum;
    datum local_key_datum;

    memset(&entry_to_return, '\0', sizeof(entry_to_return));

/* 
	Zaneme nkolika jednoduchmi kontrolami - ovme si, jestli je databze otevena a 
	jestli jsme na vstupu dostali rozumn parametry, tedy jestli vyhledvac kl 
	obsahuje platn etzec a hodnoty NULL:
*/
    if (!cdc_dbm_ptr || !cdt_dbm_ptr) return (entry_to_return);
    if (!cd_catalog_ptr) return (entry_to_return);
    if (strlen(cd_catalog_ptr) >= CAT_CAT_LEN) return (entry_to_return);

    memset(&entry_to_find, '\0', sizeof(entry_to_find));
    strcpy(entry_to_find, cd_catalog_ptr);

/*
	Pot vytvome strukturu datum, kterou ke sv innosti vyaduj funkce databze dbm, 
	a dle pomoc voln dbm_fetch nateme vlastn data. Pokud se nepodailo nast 
	dn data, vrtme przdnou strukturu entry_to_return, inicializovanou na zatku funkce:
*/
    local_key_datum.dptr = (void *) entry_to_find;
    local_key_datum.dsize = sizeof(entry_to_find);

    memset(&local_data_datum, '\0', sizeof(local_data_datum));
    local_data_datum = dbm_fetch(cdc_dbm_ptr, local_key_datum);
    if (local_data_datum.dptr) {
        memcpy(&entry_to_return, (char *)local_data_datum.dptr,
                local_data_datum.dsize);
    }
    return (entry_to_return);
} /* get_cdc_entry */

/*
	V aplikaci budeme potebovat nast tak poloku jednotliv stopy - a pesn toto 
	dl nsledujc funkce. Ta je podobn pedchoz funkci get_cdc_entry, ale do 
	parametr v n zadvme ukazatel na katalogov etzec a slo stopy:
*/

cdt_entry get_cdt_entry(const char *cd_catalog_ptr, const int track_no)
{
    cdt_entry entry_to_return;
    char entry_to_find[CAT_CAT_LEN + 10];
    datum local_data_datum;
    datum local_key_datum;

    memset(&entry_to_return, '\0', sizeof(entry_to_return));

    if (!cdc_dbm_ptr || !cdt_dbm_ptr) return (entry_to_return);
    if (!cd_catalog_ptr) return (entry_to_return);
    if (strlen(cd_catalog_ptr) >= CAT_CAT_LEN) return (entry_to_return);

    /* definovat vyhledvac kl, kter je sloenm klem 
       z poloky katalogu a sla stopy */

    memset(&entry_to_find, '\0', sizeof(entry_to_find));
    sprintf(entry_to_find, "%s %d", cd_catalog_ptr, track_no);

    local_key_datum.dptr = (void *) entry_to_find;
    local_key_datum.dsize = sizeof(entry_to_find);

    memset(&local_data_datum, '\0', sizeof(local_data_datum));
    local_data_datum = dbm_fetch(cdt_dbm_ptr, local_key_datum);
    if (local_data_datum.dptr) {
        memcpy(&entry_to_return, (char *) local_data_datum.dptr,
        local_data_datum.dsize);
    }
    return (entry_to_return);
}

// Dal funkce, add_cdc_entry, pid do katalogu novou poloku:
int add_cdc_entry(const cdc_entry entry_to_add)
{
    char key_to_add[CAT_CAT_LEN + 1];
    datum local_data_datum;
    datum local_key_datum;
    int result;

    /* zkontrolujeme, jestli je databze inicializovna
       a parametry jsou platn */
    if (!cdc_dbm_ptr || !cdt_dbm_ptr) return (0);
    if (strlen(entry_to_add.catalog) >= CAT_CAT_LEN) return (0);

    /* nyn zkontrolujeme, jestli vyhledvac kl obsahuje
       jen platn etzec a znaky null */
    memset(&key_to_add, '\0', sizeof(key_to_add));
    strcpy(key_to_add, entry_to_add.catalog);

    local_key_datum.dptr = (void *) key_to_add;
    local_key_datum.dsize = sizeof(key_to_add);
    local_data_datum.dptr = (void *) &entry_to_add;
    local_data_datum.dsize = sizeof(entry_to_add);

    result = dbm_store(cdc_dbm_ptr, local_key_datum, local_data_datum,
                    DBM_REPLACE);

    /* dbm_store() pouv 0 jako spch */
    if (result == 0) return (1);
    return (0);
}

/*
	Funkce add_cdt_entry pidv novou poloku jednotliv stopy. Sloen pstupov 
	kl tvo katalogov etzec spolu s slem stopy:
*/
int add_cdt_entry(const cdt_entry entry_to_add)
{
    char key_to_add[CAT_CAT_LEN + 10];
    datum local_data_datum;
    datum local_key_datum;
    int result;

    if (!cdc_dbm_ptr || !cdt_dbm_ptr) return (0);
    if (strlen(entry_to_add.catalog) >= CAT_CAT_LEN) return (0);

    memset(&key_to_add, '\0', sizeof(key_to_add));
    sprintf(key_to_add, "%s %d", entry_to_add.catalog,
            entry_to_add.track_no);

    local_key_datum.dptr = (void *) key_to_add;
    local_key_datum.dsize = sizeof(key_to_add);
    local_data_datum.dptr = (void *) &entry_to_add;
    local_data_datum.dsize = sizeof(entry_to_add);

    result = dbm_store(cdt_dbm_ptr, local_key_datum, local_data_datum,
                DBM_REPLACE);

    /* dbm_store() pouv 0 jako spch, 
       zporn sla jako kdy chyb */
    if (result == 0)
        return (1);
    return (0);
}

/* 
	Meme-li poloky pidvat, budeme je nkdy chtt tak odstraovat. Tato 
	funkce odstran poloku z katalogu:
*/
int del_cdc_entry(const char *cd_catalog_ptr)
{
    char key_to_del[CAT_CAT_LEN + 1];
    datum local_key_datum;
    int result;

    if (!cdc_dbm_ptr || !cdt_dbm_ptr) return (0);
    if (strlen(cd_catalog_ptr) >= CAT_CAT_LEN) return (0);

    memset(&key_to_del, '\0', sizeof(key_to_del));
    strcpy(key_to_del, cd_catalog_ptr);

    local_key_datum.dptr = (void *) key_to_del;
    local_key_datum.dsize = sizeof(key_to_del);

    result = dbm_delete(cdc_dbm_ptr, local_key_datum);

    /* dbm_delete() pouv 0 jako spch */
    if (result == 0) return (1);
    return (0);
}

/*
	A tady je analogick funkce pro odstrann jedn stopy. Pipomnme, e kl stopy 
	je sloen a e jej tvo etzec poloky katalogu plus slo stopy:
*/
int del_cdt_entry(const char *cd_catalog_ptr, const int track_no)
{
    char key_to_del[CAT_CAT_LEN + 10];
    datum local_key_datum;
    int result;

    if (!cdc_dbm_ptr || !cdt_dbm_ptr) return (0);
    if (strlen(cd_catalog_ptr) >= CAT_CAT_LEN) return (0);

    memset(&key_to_del, '\0', sizeof(key_to_del));
    sprintf(key_to_del, "%s %d", cd_catalog_ptr, track_no);

    local_key_datum.dptr = (void *) key_to_del;
    local_key_datum.dsize = sizeof(key_to_del);

    result = dbm_delete(cdt_dbm_ptr, local_key_datum);

    /* dbm_delete() pouv 0 jako spch */
    if (result == 0) return (1);
    return (0);
}

/*
	Nakonec se dostvme k jedn nemn dleit funkci, a sice jednoduch funkci 
	pro vyhledvn. Nen bhvjak dokonal, ale pkn si na n ukeme, jak 
	prochzet polokami databze dbm, ani bychom dopedu znali hodnoty kl.

	Protoe pedem nevme, kolik poloek v databzi me bt, rozhodneme se v 
	kadm voln tto funkce vrtit jedinou poloku; jakmile funkce nic nenajde, 
	vrt przdnou poloku. Pi prohledn cel databze zaneme tm, e funkci 
	vyvolme s ukazatelem na cel slo *first_call_ptr a do t pi prvnm
	vyvoln zapeme 1 (true). To znamen, e funkce zane prohledvat na 
	zatku databze; pi dalm voln bude parametr obsahovat 0 a funkce 
	bude pokraovat za posledn nalezenou polokou.

	Chcete-li spustit prohledvn znovu od zatku, teba s jinou katalogovou 
	polokou, muste funkci znovu zavolat s parametrem *first_call_ptr rovnm 1 (true).
	Mezi jednotlivmi volnmi si funkce udruje jist intern stavov informace. 
	Pro klienta je cel komplikovan mechanismus pokraovn v hledn skryt 
	a tak implementace tto funkce je "zahalena tajemstvm".
	Jestlie hledan etzec ukazuje na znak NULL, povauje se 
	jakkoli poloka za shodnou.
*/

cdc_entry search_cdc_entry(const char *cd_catalog_ptr, int *first_call_ptr)
{
    static int local_first_call = 1;
    cdc_entry entry_to_return;
    datum local_data_datum;
    static datum local_key_datum; /* tato promnn mus bt statick */

    memset(&entry_to_return, '\0', sizeof(entry_to_return));

// Jako obvykle zanme nkolika kontrolami:
    if (!cdc_dbm_ptr || !cdt_dbm_ptr) return (entry_to_return);
    if (!cd_catalog_ptr || !first_call_ptr) return (entry_to_return);
    if (strlen(cd_catalog_ptr) >= CAT_CAT_LEN) return (entry_to_return);

    /* ochrana proti situaci, kdy se nikdy neped *first_call_ptr = true */
    if (local_first_call) {
        local_first_call = 0;
        *first_call_ptr = 1;
    }

/*
	Pokud byla funkce vyvolna s parametrem *first_call_ptr = true, musme 
	zahjit (nebo znovu zahjit) hledn za zatku databze. V opanm ppad 
	jednodue pokraujeme v hledn a v databzi se pesuneme na dal kl:
*/
    if (*first_call_ptr) {
        *first_call_ptr = 0;
        local_key_datum = dbm_firstkey(cdc_dbm_ptr);
    }
    else {
        local_key_datum = dbm_nextkey(cdc_dbm_ptr);
    }

    do {
        if (local_key_datum.dptr != NULL) {
            /* poloka nalezena */
            local_data_datum = dbm_fetch(cdc_dbm_ptr, local_key_datum);
            if (local_data_datum.dptr) {
                memcpy(&entry_to_return, (char *) local_data_datum.dptr,
                        local_data_datum.dsize);
/*	
	Vlastn vyhledvn tvo velice jednoduch kontrola, pi n zjistme, 
	jestli se aktuln vyhledvac etzec vyskytuje v aktuln poloce katalogu:
*/

                /* kontrola, jestli se vyhledvac etzec vyskytuje v poloce katalogu */
                if (!strstr(entry_to_return.catalog, cd_catalog_ptr))
                {
                    memset(&entry_to_return, '\0',
                    sizeof(entry_to_return));
                    local_key_datum = dbm_nextkey(cdc_dbm_ptr);
                }
            }
        }
    } while (local_key_datum.dptr &&
            local_data_datum.dptr &&
            (entry_to_return.catalog[0] == '\0'));
    return (entry_to_return);
} /* search_cdc_entry */

