/*
	Tento soubor implementuje devt funkc pro prci s databz, jejich prototypy 
	jsou v souboru cd_data.h. Pitom pedv poadavky na server a vrac od nj 
	odpovdi a funguje tak jako prostednk. Zdrojov kd zan zalennm 
	soubor #include a definic konstant:
*/
#define _POSIX_SOURCE

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <limits.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "cd_data.h"
#include "cliserv.h"
/*
	Statick promnn mypid umouje snen jinak nezbytnho potu voln funkce 
	getpid. Duplicitnho kdu se zbavme pomoc lokln funkce read_one_response:
*/
static pid_t mypid;

static int read_one_response(message_db_t *rec_ptr);
/*
	Tak rutiny database_initialize a close pouvme i nadle, ale tentokrt slou 
	pro inicializaci klientsk strany rozhran rour a pro odstrann nepotebnch 
	pojmenovanch rour po ukonen klienta:
*/
int database_initialize(const int new_database)
{
    if (!client_starting()) return(0);
    mypid = getpid();
    return(1);
} /* database_initialize */

void database_close(void) {
    client_ending();
}
/*
	Vyvolnm rutiny get_cdc_entry nateme z databze poloku katalogu podle zadanho 
	titulu CD. Zde tak zakdujeme poadavek do struktury message_db_t a pedme jej 
	na server;odpov pak nateme zpt do jin struktury message_db_t. Je-li poloka 
	nalezena, zalen se do struktury message_db_t jako struktura cdc_entry, take 
	zpt pedvme odpovdajc st  struktury:
*/
cdc_entry get_cdc_entry(const char *cd_catalog_ptr)
{
    cdc_entry ret_val;
    message_db_t mess_send;
    message_db_t mess_ret;

    ret_val.catalog[0] = '\0';
    mess_send.client_pid = mypid;
    mess_send.request = s_get_cdc_entry;
    strcpy(mess_send.cdc_entry_data.catalog, cd_catalog_ptr);

    if (send_mess_to_server(mess_send)) {
        if (read_one_response(&mess_ret)) {
            if (mess_ret.response == r_success) {
                ret_val = mess_ret.cdc_entry_data;
            } else {
                fprintf(stderr, "%s", mess_ret.error_text);
            }
        } else {
            fprintf(stderr, "Server neodpovdl\n");
        }
    } else {
        fprintf(stderr, "Server nepijm poadavky\n");
    }
    return(ret_val);
}
/* Tady je zdrojov kd funkce read_one_response, dky n se vyhneme duplicitnmu  kdu: */
static int read_one_response(message_db_t *rec_ptr) {

    int return_code = 0;
    if (!rec_ptr) return(0);

    if (start_resp_from_server()) {
        if (read_resp_from_server(rec_ptr)) {
            return_code = 1;
        }
        end_resp_from_server();
    }
    return(return_code);
}
/*
	Ostatn rutiny get_xxx, del_xxx a add_xxx jsou implementovny podobnm zpsobem jako 
	funkce get_cdc_entry a zde si je zopakujeme jen pro plnost. Nejprve se podvejme 
	na funkci pro natn stop CD:
*/
cdt_entry get_cdt_entry(const char *cd_catalog_ptr, const int track_no)
{
    cdt_entry ret_val;
    message_db_t mess_send;
    message_db_t mess_ret;

    ret_val.catalog[0] = '\0';
    mess_send.client_pid = mypid;
    mess_send.request = s_get_cdt_entry;
    strcpy(mess_send.cdt_entry_data.catalog, cd_catalog_ptr);
    mess_send.cdt_entry_data.track_no = track_no;

    if (send_mess_to_server(mess_send)) {
        if (read_one_response(&mess_ret)) {
            if (mess_ret.response == r_success) {
                ret_val = mess_ret.cdt_entry_data;
            } else {
                fprintf(stderr, "%s", mess_ret.error_text);
            }
        } else {
            fprintf(stderr, "Server neodpovdl\n");
        }
    } else {
        fprintf(stderr, "Server nepijm poadavky\n");
    }
    return(ret_val);
}
/* Nsleduj dv funkce pro pidvn dat - nejprve do katalogu a pot do databze stop: */
int add_cdc_entry(const cdc_entry entry_to_add)
{
    message_db_t mess_send;
    message_db_t mess_ret;

    mess_send.client_pid = mypid;
    mess_send.request = s_add_cdc_entry;
    mess_send.cdc_entry_data = entry_to_add;

    if (send_mess_to_server(mess_send)) {
        if (read_one_response(&mess_ret)) {
            if (mess_ret.response == r_success) {
                return(1);
            } else {
                fprintf(stderr, "%s", mess_ret.error_text);
            }
        } else {
            fprintf(stderr, "Server neodpovdl\n");
        }
    } else {
        fprintf(stderr, "Server nepijm poadavky\n");
    }
    return(0);
}

int add_cdt_entry(const cdt_entry entry_to_add)
{
    message_db_t mess_send;
    message_db_t mess_ret;

    mess_send.client_pid = mypid;
    mess_send.request = s_add_cdt_entry;
    mess_send.cdt_entry_data = entry_to_add;

    if (send_mess_to_server(mess_send)) {
        if (read_one_response(&mess_ret)) {
            if (mess_ret.response == r_success) {
                return(1);
            } else {
                fprintf(stderr, "%s", mess_ret.error_text);
            }
        } else {
            fprintf(stderr, "Server neodpovdl\n");
        }
    } else {
        fprintf(stderr, "Server nepijm poadavky\n");
    }
    return(0);
}
/* A nakonec jsou tu dv funkce pro odstraovn dat: */
int del_cdc_entry(const char *cd_catalog_ptr)
{
    message_db_t mess_send;
    message_db_t mess_ret;

    mess_send.client_pid = mypid;
    mess_send.request = s_del_cdc_entry;
    strcpy(mess_send.cdc_entry_data.catalog, cd_catalog_ptr);

    if (send_mess_to_server(mess_send)) {
        if (read_one_response(&mess_ret)) {
            if (mess_ret.response == r_success) {
                return(1);
            } else {
                fprintf(stderr, "%s", mess_ret.error_text);
            }
        } else {
            fprintf(stderr, "Server neodpovdl\n");
        }
    } else {
        fprintf(stderr, "Server nepijm poadavky\n");
    }
    return(0);
}

int del_cdt_entry(const char *cd_catalog_ptr, const int track_no)
{
    message_db_t mess_send;
    message_db_t mess_ret;

    mess_send.client_pid = mypid;
    mess_send.request = s_del_cdt_entry;
    strcpy(mess_send.cdt_entry_data.catalog, cd_catalog_ptr);
    mess_send.cdt_entry_data.track_no = track_no;

    if (send_mess_to_server(mess_send)) {
        if (read_one_response(&mess_ret)) {
            if (mess_ret.response == r_success) {
                return(1);
            } else {
                fprintf(stderr, "%s", mess_ret.error_text);
            }
        } else {
            fprintf(stderr, "Server neodpovdl\n");
        }
    } else {
        fprintf(stderr, "Server nepijm poadavky\n");
    }
    return(0);
}

/* Tato funkce vypad komplikovanji, ne ve skutenosti je  vol toti ti funkce, 
   na kter se podvme v dal sti textu, a sice send_mess_to_server, 
   start_resp_from_server a read_resp_from_server.
*/
cdc_entry search_cdc_entry(const char *cd_catalog_ptr, int *first_call_ptr)
{
    message_db_t mess_send;
    message_db_t mess_ret;

    static FILE *work_file = (FILE *)0;
    static int entries_matching = 0;
    cdc_entry ret_val;

    ret_val.catalog[0] = '\0';

    if (!work_file && (*first_call_ptr == 0)) return(ret_val);
/*
  Zde je prvn voln operace hledn, tedy voln, pi kterm je *first_call_ptr 
  roven true. Ihned do nj zapeme false, abychom na to nezapomnli; pot vytvome 
  pracovn soubor work_file a inicializujeme strukturu klientsk zprvy.
*/
    if (*first_call_ptr) {
        *first_call_ptr = 0;
        if (work_file) fclose(work_file);
        work_file = tmpfile();
        if (!work_file) return(ret_val);

        mess_send.client_pid = mypid;
        mess_send.request = s_find_cdc_entry;
        strcpy(mess_send.cdc_entry_data.catalog, cd_catalog_ptr);
/*
  Dle pichz vnoen test t podmnek, pi kterm volme funkce ze zdrojovho souboru 
  pipe_imp.c. Pokud se zprvu poda spn odeslat na server, vyk klient doruen odpovdi. 
  Jestlie se spn poda ve voln read tak pest data zaslan serverem, vrt se do 
  souboru work_file pro pslunho klienta vyhledan shodn poloky a zv se ta entries_matching.
*/
        if (send_mess_to_server(mess_send)) {
            if (start_resp_from_server()) {
                while (read_resp_from_server(&mess_ret)) {
                    if (mess_ret.response == r_success) {
                        fwrite(&mess_ret.cdc_entry_data, 
                                sizeof(cdc_entry), 1, work_file);
                        entries_matching++;
                    } else {
                        break;
                    }
                } /* while */
            } else {
                fprintf(stderr, "Server neodpovdl\n");
            }
        } else {
            fprintf(stderr, "Server nepijm poadavky\n");
        }
/* Nsledujc test kontroluje, jestli se podailo nco najt. Pot ve voln 
  fseek nastavme pozici v souboru work_file na dal msto zpisu dat.
*/
        if (entries_matching == 0) {
            fclose(work_file);
            work_file = (FILE *)0;
            return(ret_val);
        }
        (void)fseek(work_file, 0L, SEEK_SET);
/*
	Pokud toto voln nebylo prvnm volnm funkce s tmto zadanm hledanm etzcem, 
	zkontrolujeme v dalm kdu, jestli jet zbvaj njak shody. Nakonec do struktury 
	ret_val nateme dal shodnou poloku; pedchoz kontroly zajiuj jej existenci.
*/
    } else {
        /* not *first_call_ptr */
        if (entries_matching == 0) {
            fclose(work_file);
            work_file = (FILE *)0;
            return(ret_val);
        }
    }

    fread(&ret_val, sizeof(cdc_entry), 1, work_file);
    entries_matching--;

    return(ret_val);
}

