/* Nejprve jako obvykle direktivy #include: */
#include "cd_data.h"
#include "cliserv.h"
/* Dle nadefinujeme nkolik hodnot, kter budeme potebovat v rznch funkcch: */
static int server_fd = -1;
static pid_t mypid = 0;
static char client_pipe_name[PATH_MAX + 1] = {'\0'};
static int client_fd = -1;
static int client_write_fd = -1;

/*
	Rutina server_starting vytvo pojmenovanou rouru, z n bude server st 
	pkazy ke zpracovn, a pot ji oteve pro ten. Pslun voln open zstane 
	zablokovan tak dlouho, dokud ji njak klient neoteve pro zpis. Dky 
	blokovanmu reimu bude server pi voln read vdy ekat na zasln pkaz.
*/
int server_starting(void)
{
    #if DEBUG_TRACE
        printf("%d :- server_starting()\n", getpid());
    #endif

    unlink(SERVER_PIPE);

    if (mkfifo(SERVER_PIPE, 0777) == -1) {
        fprintf(stderr, "Chyba pi spoutn serveru, roura FIFO se nevytvoila\n");
        return(0);
    }

    if ((server_fd = open(SERVER_PIPE, O_RDONLY)) == -1) {
        fprintf(stderr, "Chyba pi spoutn serveru, roura FIFO se neotevela\n");
        return(0);
    }
    return(1);
}
/*
	Jakmile server skon, odstran pojmenovanou rouru; podle toho 
	klienti poznaj, e u server nen v innosti:
*/
void server_ending(void)
{
    #if DEBUG_TRACE
        printf("%d :- server_ending()\n", getpid());
    #endif

    (void)close(server_fd);
    (void)unlink(SERVER_PIPE);
}
/*
	Funkce read_request_from_client v nsledujcm fragmentu kdu provd ten 
	ze serverov roury a zstane zablokovna, dokud do n njak klient nezape zprvu:
*/

int read_request_from_client(message_db_t *rec_ptr)
{
    int return_code = 0;
    int read_bytes;

    #if DEBUG_TRACE    
        printf("%d :- read_request_from_client()\n",  getpid());
    #endif

    if (server_fd != -1) {
        read_bytes = read(server_fd, rec_ptr, sizeof(*rec_ptr)); 

/*
	Ve specilnm ppad, kdy dn klient nem rouru otevenou k zpisu, vrt 
	voln read hodnotu 0 a to znamen, e detekuje konec souboru EOF. V takovm 
	ppad server rouru uzave a znovu ji oteve, take opt zstane zablokovan 
	do okamiku, kdy si ji oteve i njak klient. Postup je stejn jako pi prvnm 
	sputn serveru; ten jsme zde v podstat znovu inicializovali. Kd vlome 
	do pedchzejc funkce:
*/
        if (read_bytes == 0) {
            (void)close(server_fd);
            if ((server_fd = open(SERVER_PIPE, O_RDONLY)) == -1) {
               fprintf(stderr, "Server error, FIFO open failed\n");
               return(0);
            }
            read_bytes = read(server_fd, rec_ptr, sizeof(*rec_ptr));
        }
        if (read_bytes == sizeof(*rec_ptr)) return_code = 1;
    }
    return(return_code);
}

/* open the client pipe. */

/* Nyn vechny roury propojme do funknho celku. Nejprve oteveme klientskou rouru: */
int start_resp_to_client(const message_db_t mess_to_send)
{
    #if DEBUG_TRACE
        printf("%d :- start_resp_to_client()\n", getpid());
    #endif

    (void)sprintf(client_pipe_name, CLIENT_PIPE, mess_to_send.client_pid);
    if ((client_fd = open(client_pipe_name, O_WRONLY)) == -1) return(0);
    return(1);
}
/* Pomoc tto funkce budeme zaslat veker zprvy. Odpovdajc 
 klientskou funkci pro pedvn zprv si ukeme za chvli. */
int send_resp_to_client(const message_db_t mess_to_send)
{
    int write_bytes;
        #if DEBUG_TRACE
    printf("%d :- send_resp_to_client()\n", getpid());
    #endif

    if (client_fd == -1) return(0);
    write_bytes = write(client_fd, &mess_to_send, sizeof(mess_to_send));
    if (write_bytes != sizeof(mess_to_send)) return(0);
    return(1);
}
/* A nakonec klientskou rouru uzaveme: */
void end_resp_to_client(void)
{
    #if DEBUG_TRACE
        printf("%d :- end_resp_to_client()\n", getpid());
    #endif

    if (client_fd != -1) {
        (void)close(client_fd);
        client_fd = -1;
    }
}

/* Funkce client_starting si nejprve ov dostupnost serveru, 
 a pot inicializuje klientskou rouru: */
int client_starting(void)
{
    #if DEBUG_TRACE
        printf("%d :- client_starting\n", getpid());
    #endif

    mypid = getpid();
    if ((server_fd = open(SERVER_PIPE, O_WRONLY)) == -1) {
        fprintf(stderr, "Server nen v innosti\n");
        return(0);
    }

    (void)sprintf(client_pipe_name, CLIENT_PIPE, mypid);
    (void)unlink(client_pipe_name);
    if (mkfifo(client_pipe_name, 0777) == -1) {
        fprintf(stderr, "Nemohu vytvoit klientsou rouru %s\n",
                client_pipe_name);
        return(0);
    }
    return(1);
}
/* kolem funkce client_ending je uzaven souborovch deskriptor a 
 odstrann ji nepotebn pojmenovan roury: */
void client_ending(void)
{
    #if DEBUG_TRACE
        printf("%d :- client_ending()\n", getpid());
    #endif

    if (client_write_fd != -1) (void)close(client_write_fd);
    if (client_fd != -1) (void)close(client_fd);
    if (server_fd != -1) (void)close(server_fd);
    (void)unlink(client_pipe_name);
}
/* A konen funkce send_mess_to_server slou k pedvn zprv s poadavky do serverov roury: */
int send_mess_to_server(message_db_t mess_to_send)
{
    int write_bytes;

    #if DEBUG_TRACE
        printf("%d :- send_mess_to_server()\n", getpid());
    #endif

    if (server_fd == -1) return(0);
    mess_to_send.client_pid = mypid;
    write_bytes = write(server_fd, &mess_to_send, sizeof(mess_to_send));
    if (write_bytes != sizeof(mess_to_send)) return(0);
    return(1);
}

/*
	Tato klientsk funkce zane naslouchat odpovdm serveru. Oteve klientskou 
	rouru v reimu jen pro ten, a pot jej soubor oteve znovu, tentokrt 
	ale jen k zpisu. Pro, to pochopme za chvli.
*/
int start_resp_from_server(void)
{
    #if DEBUG_TRACE
        printf("%d :- start_resp_from_server()\n", getpid());
    #endif

    if (client_pipe_name[0] == '\0') return(0);
    if (client_fd != -1) return(1);

    client_fd = open(client_pipe_name, O_RDONLY);
    if (client_fd != -1) {
        client_write_fd = open(client_pipe_name, O_WRONLY);
        if (client_write_fd != -1) return(1);
        (void)close(client_fd);
        client_fd = -1;
    }
    return(0);
}
/* Zde je hlavn funkce read pro ten nalezench databzovch poloek ze serveru: */
int read_resp_from_server(message_db_t *rec_ptr)
{
    int read_bytes;
    int return_code = 0;

    #if DEBUG_TRACE
        printf("%d :- read_resp_from_server()\n", getpid());
    #endif

    if (!rec_ptr) return(0);
    if (client_fd == -1) return(0);

    read_bytes = read(client_fd, rec_ptr, sizeof(*rec_ptr));
    if (read_bytes == sizeof(*rec_ptr)) return_code = 1;
    return(return_code);
}
/* A nakonec tu mme klientskou funkci, kter vyzna konec odpovdi ze serveru: */
void end_resp_from_server(void)
{
    #if DEBUG_TRACE
        printf("%d :- end_resp_from_server()\n", getpid());
    #endif

    /* V implementaci s rourami je tato funkce przdn */
}

