/*
	Zaneme deklarac nkolika globlnch promnnch, prototypu funkce process_command 
	a funkce pro zachycovn signl, kter zajist ist ukonen programu:
*/
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <limits.h>
#include <signal.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "cd_data.h"
#include "cliserv.h"

int save_errno;
static int server_running = 1;

static void process_command(const message_db_t mess_command);

void catch_signals()
{
    server_running = 0;
}
/*
	Nyn se ji dostvme k funkci main. Po sprvnm nastaven vech rutin pro zachycen 
	signl program zkontroluje, jestli jsme na pkazovm dku pedali pznak -i. Pokud 
	ano, vytvo novou databzi; jestlie voln database_initialize ze souboru cd_dbm.c 
	sele, vype chybovou zprvu. Pokud je naopak vechno v podku a pokud se server 
	rozbhl, zanou se veker poadavky od klient zaslat do funkce process_command, 
	na kterou se podvme za chvilku.
*/
int main(int argc, char *argv[]) {
    struct sigaction new_action, old_action;
    message_db_t mess_command;
    int database_init_type = 0;

    new_action.sa_handler = catch_signals;
    sigemptyset(&new_action.sa_mask);
    new_action.sa_flags = 0;
    if ((sigaction(SIGINT, &new_action, &old_action) != 0) ||
        (sigaction(SIGHUP, &new_action, &old_action) != 0) ||
        (sigaction(SIGTERM, &new_action, &old_action) != 0)) {
        fprintf(stderr, "Chyba pi spoutn serveru, zachycen signlu selhalo\n");
        exit(EXIT_FAILURE);
    }

    if (argc > 1) {
        argv++;
        if (strncmp("-i", *argv, 2) == 0) database_init_type = 1;
    }
    if (!database_initialize(database_init_type)) {
        fprintf(stderr, "Chyba serveru:-\
                nemohu inicializovat databzi\n");
        exit(EXIT_FAILURE);
    }

    if (!server_starting()) exit(EXIT_FAILURE);

    while(server_running) {
        if (read_request_from_client(&mess_command)) {
            process_command(mess_command);
        } else {
            if(server_running) fprintf(stderr, "Server skonil \
                        - nemohu st z roury\n");
            server_running = 0;
        }
    } /* while */
    server_ending();
    exit(EXIT_SUCCESS);
}
/*
	Veker klientsk zprvy se pedvaj do ji zmnn funkce process_command; 
	zde na n "ek" pkaz case, kter zavol odpovdajc funkci ze souboru cd_dbm.c:
*/
static void process_command(const message_db_t comm)
{
    message_db_t resp;
    int first_time = 1;

    resp = comm; /* zkoprujeme pkaz zpt, pot jej podle poteby zmnme */

    if (!start_resp_to_client(resp)) {
        fprintf(stderr, "Varovn serveru:-\
            start_resp_to_client %d selhalo\n", resp.client_pid);
        return;
    }

    resp.response = r_success;
    memset(resp.error_text, '\0', sizeof(resp.error_text));
    save_errno = 0;

    switch(resp.request) {
        case s_create_new_database:
            if (!database_initialize(1)) resp.response = r_failure;
            break;
        case s_get_cdc_entry:
            resp.cdc_entry_data =
                    get_cdc_entry(comm.cdc_entry_data.catalog);
            break;
        case s_get_cdt_entry:
            resp.cdt_entry_data =
                 get_cdt_entry(comm.cdt_entry_data.catalog,
                         comm.cdt_entry_data.track_no);
            break;
        case s_add_cdc_entry:
            if (!add_cdc_entry(comm.cdc_entry_data)) resp.response =
                        r_failure;
            break;
        case s_add_cdt_entry:
            if (!add_cdt_entry(comm.cdt_entry_data)) resp.response =
                        r_failure;
            break;
        case s_del_cdc_entry:
            if (!del_cdc_entry(comm.cdc_entry_data.catalog)) resp.response = 
                        r_failure;
            break;
        case s_del_cdt_entry:
            if (!del_cdt_entry(comm.cdt_entry_data.catalog,
            comm.cdt_entry_data.track_no)) resp.response = r_failure;
            break;
        case s_find_cdc_entry:
            do {
                resp.cdc_entry_data =
                    search_cdc_entry(comm.cdc_entry_data.catalog,
                                &first_time);
                if (resp.cdc_entry_data.catalog[0] != 0) {
                    resp.response = r_success;
                    if (!send_resp_to_client(resp)) {
                        fprintf(stderr, "Varovn serveru:-\
                            neodpovd na poadavek %d\n", resp.client_pid);
                        break;
                    }
                } else {
                    resp.response = r_find_no_more;
                }
            } while (resp.response == r_success);
            break;
        default:
            resp.response = r_failure;
            break;
    } /* switch */

    sprintf(resp.error_text, "Pkaz selhal:\n\t%s\n",
            strerror(save_errno));

    if (!send_resp_to_client(resp)) {
    fprintf(stderr, "Varovn serveru:-\
            neodpovd na poadavek %d\n", resp.client_pid);
    }

    end_resp_to_client();
    return;
}

