/* Nejprve zalenme potebn hlaviky, nadefinujeme kle pro fronty 
 zprv a nadefinujeme strukturu, do n budeme ukldat data zprv: */
#include "cd_data.h"
#include "cliserv.h"

#include <sys/msg.h>

#define SERVER_MQUEUE 1234
#define CLIENT_MQUEUE 4321

struct msg_passed {
    long int msg_key; /* sem se zapisuje PID klienta */
    message_db_t real_message;
};
/* Tyto dv globln promnn budou obsahovat identifiktory front, navrcen z funkce msgget: */
static int serv_qid = -1;
static int cli_qid = -1;
/* Za vytvoen obou front zprv bude odpovdn server: */
int server_starting()
{
        #if DEBUG_TRACE
    printf("%d :- server_starting()\n", getpid());
    #endif

    serv_qid = msgget((key_t)SERVER_MQUEUE, 0666 | IPC_CREAT);
    if (serv_qid == -1) return(0);

    cli_qid = msgget((key_t)CLIENT_MQUEUE, 0666 | IPC_CREAT);
    if (cli_qid == -1) return(0);

    return(1);
}
/* Server je odpovdn tak za klid po svm ukonen. Jakmile server skon, 
 zapeme do globlnch promnnch neplatn hodnoty. Tm odchytme ppadn situace, 
 kdy by se server jet po zavoln funkce server_ending pokouel chybn odeslat zprvy:
*/
void server_ending()
{
    #if DEBUG_TRACE
        printf("%d :- server_ending()\n", getpid());
    #endif

    (void)msgctl(serv_qid, IPC_RMID, 0);
    (void)msgctl(cli_qid, IPC_RMID, 0);

    serv_qid = -1;
    cli_qid = -1;
}
/* Serverov funkce pro ten pebr z fronty zprvy libovolnho typu (tedy od 
 libovolnho klienta) a vrac z nich jen datovou st (to znamen, e typ zprvy ignoruje):
*/
int read_request_from_client(message_db_t *rec_ptr)
{
    struct msg_passed my_msg;

    #if DEBUG_TRACE
        printf("%d :- read_request_from_client()\n", getpid());
    #endif

    if (msgrcv(serv_qid, (void *)&my_msg, sizeof(*rec_ptr), 0, 0) == -1) {
        return(0);
    }
    *rec_ptr = my_msg.real_message;
    return(1);
}
/* Pi zasln odpovdi adresujeme zprvu pomoc identifiktoru 
 klientskho procesu, kter byl zapsn v poadavku:
*/
int send_resp_to_client(const message_db_t mess_to_send)
{
    struct msg_passed my_msg;

    #if DEBUG_TRACE
        printf("%d :- send_resp_to_client()\n", getpid());
    #endif

    my_msg.real_message = mess_to_send;
    my_msg.msg_key = mess_to_send.client_pid;

    if (msgsnd(cli_qid, (void *)&my_msg, sizeof(mess_to_send), 0) == -1) {
        return(0);
    }
    return(1);
}

/* Klientsk st
  Pi svm sputn mus klient zjistit identifiktory serverov a klientsk fronty; 
 sm klient ale fronty nevytv. Pokud server neb, funkce sele, protoe fronty zprv neexistuj.
*/
int client_starting()
{
    #if DEBUG_TRACE
        printf("%d :- client_starting\n", getpid());
    #endif

    serv_qid = msgget((key_t)SERVER_MQUEUE, 0666);
    if (serv_qid == -1) return(0);

    cli_qid = msgget((key_t)CLIENT_MQUEUE, 0666);
    if (cli_qid == -1) return(0);
    return(1);
}
/* Podobn jako server zape i klient po svm ukonen do globlnch promnnch 
 neplatn hodnoty. Tm odchytme ppadn situace, kdy by se klient jet 
 po zavoln funkce client_ending pokouel chybn odeslat zprvy:
*/
void client_ending()
{
    #if DEBUG_TRACE
        printf("%d :- client_ending()\n", getpid());
    #endif

    serv_qid = -1;
    cli_qid = -1;
}
/*
	Zprvu odeleme na server tak, e nejprve do pslun struktury zapeme jej data; 
	nezapomete tak piadit kl zprvy. Protoe 0 je u kle neplatnou hodnotou, take 
	pokud bychom kl ponechali nedefinovan a nabyl by zdnliv nhodn hodnoty, mohla 
	by funkce nkdy selhat, pokud bude hodnota nhodou rovna 0.
*/
int send_mess_to_server(message_db_t mess_to_send)
{
    struct msg_passed my_msg;

    #if DEBUG_TRACE
        printf("%d :- send_mess_to_server()\n", getpid());
    #endif

    my_msg.real_message = mess_to_send;
    my_msg.msg_key = mess_to_send.client_pid;

    if (msgsnd(serv_qid, (void *)&my_msg, sizeof(mess_to_send), 0) == -1) {
        perror("Odesln zprvy selhalo");
        return(0);
    }
    return(1);
}
/* Pi natn zprv ze serveru zadv klient svj vlastn identifiktor procesu a 
 pebr tak pouze zprvy, kter jsou adresovny jen jemu - zprvy pro ostatn klienty ignoruje: */
int read_resp_from_server(message_db_t *rec_ptr)
{
    struct msg_passed my_msg;

    #if DEBUG_TRACE
        printf("%d :- read_resp_from_server()\n", getpid());
    #endif

    if (msgrcv(cli_qid, (void *)&my_msg, sizeof(*rec_ptr), getpid(), 0) == -1) {
        return(0);
    }
    *rec_ptr = my_msg.real_message;
    return(1);
}
/*
	Z dvodu zachovn pln kompatibility se souborem pipe_imp.c musme nadefinovat 
	jet tyi dal funkce, kter jsou ale przdn - v ppad rour jsme je 
	implementovat museli, u front zprv u je ale nepotebujeme:
*/
int start_resp_to_client(const message_db_t mess_to_send)
{
    return(1);
}

void end_resp_to_client(void)
{
}

int start_resp_from_server(void)
{
    return(1);
}

void end_resp_from_server(void)
{
}

