/*
	Po obvyklch direktivch #include se systmovmi soubory zalenme jet soubor semun.h, 
	kter mimo jin definuje strukturu union semun, jak to vyaduje specifikace X/Open 
	(pokud ji nen definovna v systmovm souboru sys/sem.h). Dle nsleduj prototypy 
	funkc a globln promnn, po n se dostvme k funkci main. Zde ve voln semget 
	vytvome semafor a dostaneme jeho identifiktor. Je-li program vyvoln poprv (tedy 
	byl-li sputn s parametrem a je-li argc>1), provedeme voln set_semvalue pro 
	inicializaci semaforu a jako znak op_char zvolme X:
*/
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>

#include <sys/sem.h>

#include "semun.h"

static int set_semvalue(void);
static void del_semvalue(void);
static int semaphore_p(void);
static int semaphore_v(void);

static int sem_id;

int main(int argc, char *argv[])
{
    int i;
    int pause_time;
    char op_char = 'O';

    srand((unsigned int)getpid());

    sem_id = semget((key_t)1234, 1, 0666 | IPC_CREAT);

    if (argc > 1) {
        if (!set_semvalue()) {
            fprintf(stderr, "Inicializace semaforu selhala\n");
            exit(EXIT_FAILURE);
        }
        op_char = 'X';
        sleep(2);
    }
/*
	Pot pechz program do smyky, kter 10krt po sob vstoup a vystoup z kritick 
	sekce. Zde tak poprv volme funkci semaphore_p, kter semafor uzave (nastav jej 
	na ekn), protoe program se chyst vstoupit do kritick sekce:
*/
    for(i = 0; i < 10; i++) {
        if (!semaphore_p()) exit(EXIT_FAILURE);
        printf("%c", op_char);fflush(stdout);
        pause_time = rand() % 3;
        sleep(pause_time);
        printf("%c", op_char);fflush(stdout);
/*
	Po skonen kritick sekce zavolme funkci semaphore_v, kter znamen, e je semafor
	opt voln, a po nhodn dlouhm ekn se vracme do cyklu for. Kdy se tento cyklus 
	dokon, zavolme jet funkci del_semvalue, kter provede klid kdu:
*/
        if (!semaphore_v()) exit(EXIT_FAILURE);

        pause_time = rand() % 2;
        sleep(pause_time);
    }

    printf("\n%d - skonil\n", getpid());

    if (argc > 1) {
        sleep(10);
        del_semvalue();
    }
    exit(EXIT_SUCCESS);
}

// Funkce set_semvalue provd inicializaci semaforu - k tomu ve voln semctl 
// zadv pkaz SETVAL. Bez inicializace bychom nemohli semafor pouvat:
static int set_semvalue(void)
{
    union semun sem_union;

    sem_union.val = 1;
    if (semctl(sem_id, 0, SETVAL, sem_union) == -1) return(0);
    return(1);
}
// Velmi podobnou strukturu m i funkce del_semvalue, kter ale ve voln 
// semctl poaduje namsto pkazu SETVAL operaci IPC_RMID a tm i odstrann 
// pslunho ID semaforu:
static void del_semvalue(void)
{
    union semun sem_union;

    if (semctl(sem_id, 0, IPC_RMID, sem_union) == -1)
        fprintf(stderr, "Odstrann semaforu selhalo\n");
}
// Dal funkce, semaphore_p, je implementac operace P neboli wait; 
// zmn proto hodnotu semaforu o -1:
static int semaphore_p(void)
{
    struct sembuf sem_b;

    sem_b.sem_num = 0;
    sem_b.sem_op = -1; /* P() */
    sem_b.sem_flg = SEM_UNDO;

    if (semop(sem_id, &sem_b, 1) == -1) {
        fprintf(stderr, "semaphore_p selhalo\n");
        return(0);
    }
    return(1);
}
// Posledn semaphore_v je opt podobn, ale do poloky sem_op struktury 
// sembuf zapisujeme 1 a provdme tm uvolnn semaforu neboli operaci V (signal):
static int semaphore_v(void)
{
    struct sembuf sem_b;

    sem_b.sem_num = 0;
    sem_b.sem_op = 1; /* V() */
    sem_b.sem_flg = SEM_UNDO;

    if (semop(sem_id, &sem_b, 1) == -1) {
        fprintf(stderr, "semaphore_v selhalo\n");
        return(0);
    }
    return(1);
}

